/**
  * rmFxIcalMail.js
  *
  *	SEND:
  *	  Reminderfox events stored in the users list can be send using the TB/mail facility.
  *	  Sending modes are:
  *	  - just send one or more events from the list to another
  *				user, this is the METHOD:PUBLISH.
  *	  - send one event as "Invitation" or "Schedule", this is
  *				based on RFC2446 (iTIP)
  *	  Current implementation supports:
  *	  -- METHOD: PUBLISH, REQUEST, REPLY
  *
  *	RECEIVE:
  *	  ICS-file with RFC2445 (iCalendar) data is read and added
  *			to the Reminderfox.ics file.
  *	  The ICS-data can be part of the body or attached as an ICS file.
  *	  The data is interpreted in conformance to RFC2446 and supports:
  *		-- METHOD: PUBLISH, REQUEST, REPLY, CANCEL
  *
  *	NOTE:
  *		Both RFC's are not covered in all aspects, for actual implementation
  *			please refer to  'RmFx_iCal_iTIP_Implementation.pdf'
  *
  *	@since 2008-04-14
  *	  - messageID added to event. This enables to call msg from reminder
  *				and reminder from msg;
  *	  - msg tagging with iCal METHOD;
  *	  - D&D added;
  *	  - Attendee can process CANCEL send by Organizer;
  *	  - supports reminderfox.util.PromptAlert(msg) instead of plain 'alert';
  *
  *	@since 2009-03 D&D:  moved to  'utils/rmFxUserIO.js'
  *
  *	@see  'RmFx_iCal_iTIP_Implementation.pdf'
  **/
if(!reminderfox)		var	reminderfox = {};
if(!reminderfox.iCal)		reminderfox.iCal = {};
if(!reminderfox.util)		reminderfox.util ={};


reminderfox.iCal.fromMailadr = null;
//  reminderfox.iCal.method;  ==> .core.
reminderfox.iCal.attendees = null;
reminderfox.iCal.organizer = null;
reminderfox.iCal.messageId = null;
reminderfox.iCal.lastEvent = null;
reminderfox.iCal.selectedReminders = null;


/**
 * An 'Add Reminder' with Context Menu on 3pane and Attachment pane    <br>
 * 
 * Called from reminderFoxOverlay.xul context menu with  <br>
 *   event.id =   "mailRm_ICS_List"  for attachment
 *   event.id =   "mailRm_ICS_Msg"   for message
 *  
 *  @param {object}  event 
 *  
 */
reminderfox.iCal.getiCalwCMenu = function (event) {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	var msgHdr = gDBView.hdrForFirstSelectedMessage;
	var uri = msgHdr.folder.getUriForMsg(msgHdr);

	var msgDetails =reminderfox.util.selectionDetails(msgHdr, "");

	// (event.id == "mailRm_ICS_Msg")   // call is message
	// (event.id == "mailRm_ICS_List")  // call is from attachment 

	reminderfox.iCal.addReminder4Message (uri, msgDetails, event.id /*info*/) 
}


/**
 *  Get the selected attachments if messagePane id open
 * 
 * 	@return  {object}  thisAttachments  or null
 */
reminderfox.iCal.getAttachments = function () {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	var attachmentList = document.getElementById("attachmentList")
	return /*thisAttachments */ ((!attachmentList.parentNode.collapsed) && (attachmentList.selectedCount > 0)) 
		? attachmentList.selectedItems : null;
}


/**
 *  "Add Reminder" for *selected* message/attachment(s) <br>
 *
 *  Check message body and all/selected attachments for iCal/ICS data.   <br>
 *  For any 'single' event/todo open the "Add Reminder" dialog.   <br>
 *  For multiple events/todos in body or attachment excecute as "Import".   <br>
 *  - If NO iCal/ICS data found, build [Mail] reminder.
 * 
 * @param {Object} dropData
 * @param {Object} msg
 * @param {Object} info
 *  
 */
reminderfox.iCal.addReminder4Message = function (dropData, msg, info) {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
reminderfox.util.dump2Console ('refactor',
		"((" + (info += "% addReminder4Message " + dropData) + "))", Components.stack.caller);  //gWrefactor

	reminderfox.core.method = "";
	try {
		document.getElementById('threadPaneContext').hidePopup();
	} catch (e) {}
	try {
	document.getElementById('messagePaneContext').hidePopup();
	} catch (e) {}

	var msgHdr = gDBView.hdrForFirstSelectedMessage;
	
	var thisAttachments = reminderfox.iCal.getAttachments();
	
	var dd = (thisAttachments == null);  // if not from 'Context menu' or no Attachments 
	var numAttachments = (thisAttachments == null) ? -1 : thisAttachments.length;
	
reminderfox.util.dump2Console ('refactor', 'addEmailReminders  thisAttachments.len : ' + numAttachments);

	var iCalAttachmentFlag = false;
	
	// XXX  TODO  this will fail with "news://"						+++2todo +++2todo +++2todo
	MsgHdrToMimeMessage(msgHdr, null, function (aMsgHdr, aMimeMsg) {
		aAttachments = aMimeMsg.allUserAttachments || aMimeMsg.allAttachments;

reminderfox.util.dump2Console ('refactor', 'addEmailReminders  aAttachments.len : ' + aAttachments.length);

		if ((numAttachments == -1) && (aAttachments.length != -1)){
			numAttachments = aAttachments.length
		}

reminderfox.util.dump2Console ('refactor', 'addEmailReminders  msg\n>"' + msgHdr.mime2DecodedSubject +'"<'
				+ "\n numAttachments :" + numAttachments);
		var msgDetails = reminderfox.util.selectionDetails(msgHdr);

		var i = 0;
		while (i < (numAttachments)) {

			var _contentType;
			var url;

			if (dd)	_contentType = aAttachments[i].contentType.toLowerCase();
			if (!dd)	_contentType = thisAttachments[i].attachment.contentType.toLowerCase();

reminderfox.util.dump2Console ('refactor', "\nattachment  i:" + i + "  type: " + _contentType);

			if ( (_contentType == "text/calendar")                   /* 'normal' apps */
				|| (_contentType == "application/ics")                /*  Google Calendar */
				|| (_contentType == "application/octet-stream")) {    /* some calendars do not identify as calendar type */

				iCalAttachmentFlag = true;
				var iOService = Components.classes["@mozilla.org/network/io-service;1"]
									.getService(Components.interfaces.nsIIOService);

				if (dd)		url = iOService.newURI(aAttachments[i].url, null, null);
				if (!dd)		url = iOService.newURI(thisAttachments[i].attachment.url, null, null);

				let channel = iOService.newChannelFromURI(url);

				let chunks = [];
				let aData = "";
				let listener = {
					setMimeHeaders: function () {
					},

					onStartRequest: function (/* nsIRequest */ aRequest, /* nsISupports */ aContext) {
					},

					onStopRequest: function (/* nsIRequest */ aRequest, /* nsISupports */ aContext, /* int */ aStatusCode) {
						let aData = chunks.join("");
						// check for iCal/ICS -- if true process as 'Add Reminder'
						if((aData.indexOf("BEGIN:VCALENDAR") != -1) &&
								((aData.indexOf("VEVENT") != -1) || (aData.indexOf("VTODO") != -1))){
							info = " %msgAttachment# " + i;
							reminderfox.iCal.getiCalFromString(aData, msgDetails.summary, info);
							
						}
					},

					onDataAvailable: function (/* nsIRequest */ aRequest, /* nsISupports */ aContext,
						/* nsIInputStream */ aStream, /* int */ aOffset, /* int */ aCount) {
						var scriptableStream = Components.classes["@mozilla.org/scriptableinputstream;1"]
								.createInstance(Components.interfaces.nsIScriptableInputStream);
						scriptableStream.init(aStream);
						var data = scriptableStream.read(scriptableStream.available());
						let unicodeConverter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"]
										.createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
						unicodeConverter.charset = "UTF-8";
						// So charCodeAt is what we want here...
						let array = [];
						for (let i = 0; i < data.length; ++i)
							array[i] = data.charCodeAt(i);
						chunks.push(unicodeConverter.convertFromByteArray(array, array.length));
					}
				}; // listener
				channel.asyncOpen(listener, null);
			} // check for iCal attachment content
			i++;
		}; // while numAttachments

reminderfox.util.dump2Console ('refactor', "MsgHdrToMimeMessage  iCalAttachmentFlag(1):" + iCalAttachmentFlag);

		var msgDetails = reminderfox.util.selectionDetails(msgHdr);

		var msgBody = reminderfox.iCal.getMessageBody(msgHdr);
		if((msgBody.indexOf("BEGIN:VCALENDAR") != -1) &&
				((msgBody.indexOf("VEVENT") != -1) || (msgBody.indexOf("VTODO") != -1))){

			iCalAttachmentFlag = true
			var info = " 'Import iCal data' received with a message containing iCal data ";
			var msg = " [" + reminderfox.core.getBundle().getString("rf.schedule.import.dand") + "]";
			reminderfox.iCal.getiCalFromString(msgBody, msg, info);
		}

		// "\n ..[mail]  --> 'Add Reminder' dialog")

		 // if 'iCalAttachmentFlag' check is disabled the message body is ALWAYS processed
		 // with enabled, the body will only be processed if none of the attachments has ICS data
		if (!iCalAttachmentFlag) {

				var newDate = new Date();
				newDate.setDate( newDate.getDate() + 1 );  // default to using tomorrow's date for reminder
				var reminderId =reminderfox.core.generateUniqueReminderId( newDate );
				
				var newReminderToBeAdded = new reminderfox.core.ReminderFoxEvent( reminderId, newDate, msgDetails.infos.subject );				
				newReminderToBeAdded.messageID = msgDetails.messageId;
				newReminderToBeAdded.priority = msgDetails.priority;
				newReminderToBeAdded.notes = msgDetails.text;

				reminderfox.iCal.tagging(reminderfox.core.addReminderHeadlessly(newReminderToBeAdded), "", "");
		} // iCalAttachmentFlag
	}); // MsgHdrToMimeMessage
	
	var aa = 0;
};


/**
 * Read the message body to a stream
 * 
 * @param {msgHdr} aMessageHeader
 */
reminderfox.iCal.getMessageBody = function (aMessageHeader){
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	let messenger = Components.classes["@mozilla.org/messenger;1"]
		.createInstance(Components.interfaces.nsIMessenger);
	let listener = Components.classes["@mozilla.org/network/sync-stream-listener;1"]
		.createInstance(Components.interfaces.nsISyncStreamListener);
	let uri = aMessageHeader.folder.getUriForMsg(aMessageHeader);

	messenger.messageServiceFromURI(uri)
		.streamMessage(uri, listener, null, null, false, "");

	let folder = aMessageHeader.folder;
	return folder.getMsgTextFromStream(listener.inputStream,
		aMessageHeader.Charset,
		65536,
		32768,
		false,
		true,
		{ });
};


/**
 *  Reads iCal/ICS from a string and converts to event and todo arrays
 *    @param  {string}  _content
 *    @param  {string}  mInvitation
 *    @param  {string}  info  about caller/function
 *    @return {boolean} rv =  true with sucess
 * 
 * /////////// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! \\\\\\\\\\\\\\\
 * @since  2011-03-31   
 * *** EXPERIMENTAL ONLY *** NO CHANGE TO NORMAL Reminderfox code base *****
 * 	jquery/icalendar to parse the ICS-string
 *		$.icalendar.parse(_content)   parses to:
 *			{array} $icalendar.ics
 * 			{array} vcalendar
 * 				{array} vevent
 * 				{array} vtodo
 * 				{array} timezone
 */
reminderfox.iCal.getiCalFromString = function (_content, mInvitation, info) {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
reminderfox.util.dump2Console ('refactor',
		"((" + (info += " %FromString") + "))",  Components.stack.caller);  //gWrefactor

/*---------------
	$.icalendar.parse(_content);
	var vCalendar = $.icalendar.ics.vcalendar;

	if ($.icalendar.errors[0] != null) reminderfox.util.dump2Console ('refactor',
		"((" + (info += " %FromString") 
		+ "\n **** The iCal/ICS data has error(s) *** " + $.icalendar.errors
		+ "))");  //gWrefactor
------------*/

	// if no iCAL/ICS data return with 'false'
	if (!((_content.indexOf("BEGIN:VCALENDAR") != -1) &&
			((_content.indexOf("VEVENT") != -1) || (_content.indexOf("VTODO") != -1)))){
		return false;
	}

	var _reminderEvents = new Array();
	var _todosArray = { };
	reminderfox.core.readInRemindersAndTodosICSFromString(_reminderEvents, _todosArray, _content, true);

	// need to remember 'METHOD' from the string to tag the message correctly!
	var method = reminderfox.core.method;

	reminderfox.iCal.tagging(reminderfox.iCal.getEvents(mInvitation, _reminderEvents, _todosArray, info), method, info);
	return true;
};


/**
 * 	Tag a message
 * 
 * @param {Boolean} rv
 * @param {object}  tag  null --> 'Reminderfox <br>
 *                  string --> iCal-(string); eg. iCal-Reply
 * @param {string} info
 */
reminderfox.iCal.tagging = function (rv, tag, info) {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	if (reminderfox.util.messenger()) {
reminderfox.util.dump2Console ('refactor',
	"  ((" + (info = " %iCal.tagging") + "))" + tag, Components.stack.caller);  //gWrefactor
	
		if ((tag == null) && (rv == true)){
			reminderfox.tagging.msg('Reminderfox', true, '#3333FF');
			return;
		}
		if ((tag != "") && (rv == true)){
			reminderfox.tagging.msg('iCal-' + tag, true, '#3333FF');
		}
	}
};


/**
 *  Get all Events/Todos from 
 *  
 *    @param  {string}  mInvitation
 *    @param  {array}   _reminderEventsArg
 *    @param  {array}   _todosArrayArg
 *    @param  {string}  info  about caller/function
 *    @return {boolean} rv =  should be true if added to list, so tag the msg
 */
reminderfox.iCal.getEvents = function(mInvitation, _reminderEventsArg, _todosArrayArg, info) {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
reminderfox.util.dump2Console ('refactor',
	 "(("+ (info += " %getEvents") + ")) " +reminderfox.core.method, Components.stack.caller);	//gWrefactor


	//  here we need to know the METHOD of the event
	//	 with "send"	 it's set by processing before
	//	 with "receive" it's written to "reminderfox.core.method;"
	var method = reminderfox.core.method;

reminderfox.util.dump2Console ('refactor',
	"(("+ (info += " %getEvents(1)" + method) + ")) ", Components.stack.caller);	//gWrefactor

	// now iterate over each VEVENT, for Invitations
	// get infos into reminder.notes ... depended of "METHOD"
	var numEvents = _reminderEventsArg.length;
	for ( var i = 0; i < numEvents; i++ ) {
		var reminder = _reminderEventsArg[i];

		// prevent to import reminders without UID .. as they may come from Outlook
		if (reminder.id == null) {
			reminder.id = reminderfox.core.generateUniqueReminderId() ;
		}
		var result = reminderfox.iCal.processExtraScheduleInfo(reminder, mInvitation, method );
		if (result != "") {
			reminderfox.util.PromptAlert(result);
			return false;
		}
	}
	// count the VTODOs
	var numTodos = 0;
	for ( var list in _todosArrayArg ) {
		var cList = _todosArrayArg[list];
		for (var next in cList) {
			numTodos++;
		}
	}
	var multipleEventsOrTodo =  ((numEvents > 1) || (numTodos > 0)) ? true : false;

	// #1	handle multiple VEVENT  OR  VTODO -->>  import them all with normal import

	if (multipleEventsOrTodo) {
		// check [Revert] status
		// import with tagging the 'CATEGORIES' item, ask user / get prefs

		var params = { mode: 'CANCEL',
			categories:"importICS",
			numEvents:numEvents,
			numTodos:numTodos,
			info: "## " + info };

		window.openDialog("chrome://reminderfox/content/categories/rmFxImportWithCategory.xul",
		"rmFx_ReminderImportwCategory", "chrome,resizable,modal", params);
		// mode :  CANCEL or OK
		if (params.mode == 'CANCEL'){ return 'CANCEL'} //  false;}

		// ...  add CATEGORY item for import
		var categoryItem = params.categories;

		for ( var i = 0; i < _reminderEventsArg.length; i++ ) {
			reminder = reminderfox.util.addCategory4Import (_reminderEventsArg[i], categoryItem);
		}
		for ( var list in _todosArrayArg ) {
			var cList = _todosArrayArg[list];
			for (var next in cList) {
				reminderfox.util.addCategory4Import (cList[next], categoryItem);
			}
		}

		var rv = reminderfox.util.addMultipleEvents(_reminderEventsArg, _todosArrayArg);
		return rv;
	}

	// #2a  single Event/Schedule.
	//		 Check for Reminders passed. None --> return false;
	var reminder = _reminderEventsArg[0];	// the 'read' reminder
	if (reminder == null) return false;	 // no Reminders, return

	/* XXX      +++2ToDo   ????????????
	*---------------------------------
	// check if previous event is the same as this one (POSTBOX workaround)
	try {
		if ((reminder.id == reminderfox.iCal.lastEvent.id)
			&& (reminder.extraInfo == reminderfox.iCal.lastEvent.extraInfo)) {
			// reminderfox.util.PromptAlert(" here we have an duplicated event");
			reminderfox.iCal.lastEvent = null;
			return false;
		}
	} catch (ex) {}
	-------------------------------- */
	reminderfox.iCal.lastEvent = reminder;

	// #2b	loop with all reminders in list and check if already in reminder.ics
	//		 known: return false
	//		 new:	call the "Add Reminder" dialog, return "true" if finally added,
	//			 "false" if user suspend adding
	var statusMessage="";
	reminder.messageID = reminderfox.iCal.messageId;

	var newExtra = reminderfox.core.extraInfoArray(reminder);  // the 'extraInfo' items

	var activeReminders = reminderfox.core.getReminderEvents();

	for (var nR =0; nR < activeReminders.length; nR++) {
		if (reminder.id == activeReminders[nR].id ) {
			// reminder already known, check for details and alert it.

			var oldExtra = reminderfox.core.extraInfoArray(activeReminders[nR]);
			var same  = (reminderfox.core.extraInfo(newExtra,"DTSTAMP"))
						== (reminderfox.core.extraInfo(oldExtra,"DTSTAMP"))
						&& (reminderfox.core.extraInfo(newExtra,"SEQUENCE"))
						== (reminderfox.core.extraInfo(oldExtra,"SEQUENCE")) ;	  //++++2do  ++++2do  ++++2do  ++++2do  ++++2do
																		//  check for oldSEQ > newSEQ  ==> ERR
			var older = (reminderfox.core.extraInfo(newExtra,"DTSTAMP"))
						<= (reminderfox.core.extraInfo(oldExtra,"DTSTAMP"))
						|| (reminderfox.core.extraInfo(newExtra,"SEQUENCE"))
						 < (reminderfox.core.extraInfo(oldExtra,"SEQUENCE")) ;	  //++++2do  ++++2do  ++++2do  ++++2do  ++++2do
																	//		  for oldDTSTAMP > new ==> ERR

			switch (method) {

				case "CANCEL":  {
					// check if msg/cancel was already processed
					if  (same == true) {
						statusMessage =  "[iCal  CANCEL] " + ' "' + activeReminders[nR].summary + '"'
										+  "\n\n" + mInvitation
										+  "\n\n" + reminderfox.core.getBundle().getString("rf.schedule.attendee.processed");
						reminderfox.util.PromptAlert(statusMessage);
						return false;
					}
					if  (older == true) {
						statusMessage =  "[iCal  CANCEL] " + ' "' + activeReminders[nR].summary  + '" '
										+  "\n\n" + mInvitation
										+  "\n\n" + reminderfox.core.getBundle().getString("rf.schedule.older1.processed")
										+  "\n"   + reminderfox.core.getBundle().getString("rf.schedule.older2.processed") ;
						reminderfox.util.PromptAlert(statusMessage);
						return false;
					}
					// save all event-items {reminder.xx} if not included
					// in CANCEL-msg {activeReminders[nR].xx}
					//	 reminder.categories, reminder.location, reminder.url, reminder.priority,
					// also set Description to [iCal CANCEL] ....
					//
					if (reminder.categories == null) {reminder.categories = activeReminders[nR].categories};
					if (reminder.location == null) {reminder.location = activeReminders[nR].location};
					if (reminder.url == null) {reminder.url = activeReminders[nR].url};
					if (reminder.priority == null) {reminder.priority = activeReminders[nR].priority};
					reminder.notes = reminder.notes +'\n\n' +  activeReminders[nR].notes;

					break;
				}

				case "REQUEST":  {
					// check if msg/reply was already processed
					if (same == true) {
						statusMessage = "[iCal  REQUEST] " + ' "' + activeReminders[nR].summary + '" '
									+ "\n\n" + mInvitation
									+ "\n\n" + reminderfox.core.getBundle().getString("rf.schedule.attendee.processed");
						reminderfox.util.PromptAlert(statusMessage);
						return false;
					}
					if (older == true) {
						statusMessage =  "[iCal  REQUEST] " + '" ' + activeReminders[nR].summary + '" '
									+ "\n\n" + mInvitation
									+ "\n\n" + reminderfox.core.getBundle().getString("rf.schedule.older1.processed")
									+ "\n" + reminderfox.core.getBundle().getString("rf.schedule.older2.processed");
						reminderfox.util.PromptAlert(statusMessage);
						return false;
					}
					break;
				}

				case "REPLY":  {
					// check if msg/reply was already processed.
					// result on 'same' only doesn't say if the current
					// processed msg was already processed.
					// So we check if a TAG 'iCal-Reply' is added to the msg.

					// this.currentHeaderData.tags.headerValue
					var msgHdr = gDBView.hdrForFirstSelectedMessage;
					var msgKeys = msgHdr.getStringProperty("keywords");
					if	((same == true) && (msgKeys.indexOf("iCal-REPLY") > -1)) {

						var msg =  "[REPLY] " + ' "' + activeReminders[nR].summary + '" '
									+ "\n\n" + mInvitation
									+ "\n\n" + reminderfox.core.getBundle().getString("rf.schedule.attendee.processed");
						if (confirm(msg + "\n\n" + reminderfox.core.getBundle().getString("rf.schedule.processReply")) == false) {
							return false;
						}
					}

					var attNo;

					reminderfox.iCal.fromMailadr = msgHdr.mime2DecodedAuthor;

					var fromAttendee = reminderfox.iCal.fromMailadr.substring(
								reminderfox.iCal.fromMailadr.indexOf( "<" ) + 1,
								reminderfox.iCal.fromMailadr.indexOf( ">" ));
					var fromATT = fromAttendee.toUpperCase();

					for ( var i = 0; i < reminderfox.iCal.attendees.length; i++ ) {
						if (reminderfox.iCal.attendees[i].toUpperCase().indexOf(fromATT) > -1) attNo=i;
					}
					// is the "from" address == the ATTENDEE ??,
					// if NOT  --> unknown CU ( + a "Party Crasher") ...
					//  So we have to ask the user which ATTENDEE has to be taken here
					if (reminderfox.iCal.attendees[attNo] == null) {

						var results = { attendees:reminderfox.iCal.attendees };
						window.openDialog("chrome://reminderfox/content/mail/rmFxReplyAttendees.xul",
						"rmFx-set-getReplyAttendee", "chrome,resizable,modal", results);
						// CANCEL and not seleteced ==  -1
						// OK and selected == #
						if (results.attendeeNo == -1){
							return false;}

						var xx= reminderfox.iCal.attendees[results.attendeeNo];

						var fndIndex = xx.toUpperCase().lastIndexOf( "MAILTO:" );
						if (fndIndex == -1) { return false};
							var fromATT =  xx.substring( fndIndex + 7).toUpperCase();
					}
					// clone the stored event and update some items from
					// newEvent [REPLY] to oldEvent [the stored]
					//  - save the .notes
					var replyReminderNotes = reminder.notes;

					//  - ATTENDEE  grap the new
					for ( var i = 0; i < newExtra.length; i++ ) {
						if (newExtra[i].toUpperCase().indexOf(fromATT) > -1) var iNew=i;
					}
					for ( var i = 0; i < oldExtra.length; i++ ) {
						if (oldExtra[i].toUpperCase().indexOf(fromATT) > -1) var iOld=i;
					}
					oldExtra[iOld] = newExtra[iNew];

					// rebuild the extraInfo string
					for ( var i = 0; i < oldExtra.length; i++ ) {
						if (i==0) {activeReminders[nR].extraInfo = oldExtra[i];
						} else {
						activeReminders[nR].extraInfo += "\\n" + oldExtra[i];}
					}

					reminder = activeReminders[nR];
					reminder.notes = replyReminderNotes
						+ "\n _._._._._._._ " + reminderfox.core.getBundle().getString("rf.schedule.previous") + " _._._._._._._\n"
						+ activeReminders[nR].notes;
					break;
				}  //METHOD:REPLY

				default: {//METHOD: ... all other (for the moment)
					  //  get info from reminder 'in the list'

					var rSEQ = reminderfox.core.extraInfo(oldExtra,'sequence')
					var rDTSTAMPz = reminderfox.core.extraInfo(oldExtra,'dtstamp');

					var rDTSTAMP = "";
					if (rDTSTAMPz != null ) {
							 rDTSTAMP= reminderfox.date.parseDateTimes(rDTSTAMPz) ;
					}
					//  get info from 'new' reminder
					var nSEQ = reminderfox.core.extraInfo(newExtra,'sequence');
					var nDTSTAMPz = reminderfox.core.extraInfo(newExtra,'dtstamp');

					var nDTSTAMP = "";
					if ( nDTSTAMPz != null ) {
							 nDTSTAMP =  reminderfox.date.parseDateTimes(nDTSTAMPz) ;
					}
					var newStr = reminderfox.core.getBundle().getString("rf.schedule.new");
					var oldStr = reminderfox.core.getBundle().getString("rf.schedule.old");

					var info1 = reminderfox.core.getBundle().getString("rf.reminder") + " [ " + activeReminders[nR].summary + " ] \n"
						+reminderfox.core.getBundle().getString("rf.schedule.check.summary")+ nR + "\n\n";

					var info2 =  "\n\nSEQUENCE\n	 "+ newStr +":  [#"	  + nSEQ		+ "]	  "+ oldStr +": [#" + rSEQ
						+	"]\nDTSTAMP \n	  "+ newStr +":  [" + nDTSTAMP + "]	  "+ oldStr +":  [" + rDTSTAMP + "]";

					if ((nSEQ *1) > (rSEQ*1)) { // new SEQUENCE should be greater
						statusMessage = reminderfox.core.getBundle().getString("rf.schedule.new") + " '" + reminderfox_Method + "' "
										+ " SEQUENCE" + " #" + nSEQ ;
				//	alert (info1  + statusMessage +  info2) ;
					} else {
						if (nDTSTAMPz  <= rDTSTAMPz  ) {
							statusMessage = 'SEQUENCE / DTSTAMP ' + reminderfox.core.getBundle().getString("rf.schedule.notValid");
							reminderfox.util.PromptAlert(info1  + statusMessage +  info2) ;
							return false;

						} else {
							statusMessage = reminderfox.core.getBundle().getString('rf.schedule.newDate') + '  [' + nDTSTAMP + ']';
							//				  alert (info1  + statusMessage +  info2) ;
						}
					}

					if (statusMessage != ""  && activeReminders[nR].notes != null)  {
						reminder.notes += "\n _._._._._._._ "
											+ reminderfox.core.getBundle().getString("rf.schedule.previous")
											  +  " _._._._._._._\n" +
						activeReminders[nR].notes
					}
				} // METHOD  ... all other
			} // switch
		 } // reminder already known
	 } //#2a	-- loop with all reminders in  list

	var rv = reminderfox.core.addReminderHeadlessly( reminder, true, true );  // gW 2009-02-25 edit flag added

reminderfox.util.dump2Console ('refactor',
	 "(("+ (info += " %addReminderHeadlessly ") + rv + " )) " +reminderfox.core.method, Components.stack.caller);	//gWrefactor

	return rv;  // should be true if added to list, so tag the msg
};

/**
 *   Reads the '.extraInfo' of a reminder to prepare the reminder.notes 
 *   which reflects the schedule items, 
 *   on error situations return message string
 *   
 *    @param  {array}  reminder:     current reminder
 *    @param  {string} mInvitation   string
 *    @param  {string} aMethod:  REPLY, REQUEST, CANCEL, COUNTER, DECLINECOUNTER
 */
reminderfox.iCal.processExtraScheduleInfo = function (reminder, mInvitation, aMethod ) {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	var msgReturn="";

	if ( reminder.extraInfo == null ) return msgReturn;

//  here we need to know the METHOD of the event
//    it's written to "this.reminderfox_Method"
//     eg.  REPLY, REQUEST, CANCEL, COUNTER, DECLINECOUNTER

	//var this.method = this.reminderfox_Method;

	var addPerson = "";
	var addOption="";  var iSchComment = null;  var iSchStatus = null;
	var nDTSTAMP;      var iSchStart;           var iSchEnd;
	var option ="";
	var ret = "\n";

	var extraInfos =  reminderfox.core.extraInfoArray(reminder);

	var nDTSTAMPz = reminderfox.core.extraInfo(extraInfos,"DTSTAMP");
	if (nDTSTAMPz != "" ) {
		nDTSTAMP  =  reminderfox.date.parseDateTimes(nDTSTAMPz);
	}
	var nSEQ = reminderfox.core.extraInfo(extraInfos,"SEQUENCE");

	option = reminderfox.core.extraInfo(extraInfos,"COMMENT");
	if (option != "") iSchComment = option + ret;

	// next items go with  'addOption' to Notes
			function addOptItem (option, optText) {
				// var option = reminderfox.core.extraInfo(extraInfos,"PRIORITY");
				return ((option == null) || (option == ""))
					? ""
					: "	 " + optText + " [" + option +  "]";
			}
	addOption += addOptItem (reminderfox.core.extraInfo(extraInfos,"PRIORITY"), "Priority");
	addOption += addOptItem (reminderfox.core.extraInfo(extraInfos,"CLASS"), "Class");
	addOption += addOptItem (reminderfox.core.extraInfo(extraInfos,"CUTYPE"), "CuType");
	addOption += addOptItem (reminderfox.core.extraInfo(extraInfos,"TRANSP"), "TRANSP");

	// ---->>  process PERSONS infos  ------
	var myMailAccounts = reminderfox.msgnr.myMailIDs();

	reminderfox.iCal.organizer = reminderfox.core.extraInfo(extraInfos,"ORGANIZER", ";");

	// check if ORGANIZER is part of my accounts,
	//	 if METHOD:REPLY  it has to be! Else ERROR, don't add to reminder.ics
	if (aMethod == "REPLY") {
		var fndIndex = reminderfox.iCal.organizer.toUpperCase().lastIndexOf( "MAILTO:" );
			if (fndIndex != -1) {
			var fndString = "<" + reminderfox.iCal.organizer.substring( fndIndex + 7) + ">";
			if (myMailAccounts.indexOf(fndString) == -1 ) {
				msgReturn = "[REPLY] " +reminderfox.core.getBundle().getString("rf.schedule.replyWrongOrganizer")
					+  ": \n" + fndString;
				return msgReturn;
			}
		}
	}

	// get PARTSTAT for myAccount
	var _Attendees = reminderfox.core.extraInfo(extraInfos,"ATTENDEE", ";", true);  // true==read multiple entries
	var attPart ="";

	reminderfox.iCal.attendees = _Attendees.split("\n");

	if (reminderfox.iCal.attendees.length > 0  || nSEQ != null){
		for (var n=0; n<reminderfox.iCal.attendees.length; n++){
			reminderfox.iCal.attendees[n] = reminderfox.iCal.attendees[n].toUpperCase();
			var fnd =  reminderfox.iCal.attendees[n].lastIndexOf("MAILTO:");
			if (fnd != -1) {
				var address = reminderfox.iCal.attendees[n].substring(fnd+7);
				if (address.indexOf(myMailAccounts)) {

					fnd =  reminderfox.iCal.attendees[n].indexOf("PARTSTAT=");
					if (fnd != -1) {
						var attPart = reminderfox.iCal.attendees[n].substring(fnd+9);
						fnd = attPart.indexOf(";");
						if (fnd == -1) {attPart.indexOf(":")};
						var attPart = attPart.substring(0,fnd);
					}
				}
			}
		}
	}
	var aPART ="";
	if (aMethod != "REQUEST") { aPART = "  " + attPart; }
	var extraInfoSchedule = "[iCal] " + aMethod + aPART + ": "
			+ reminderfox.core.getBundle().getString("rf.schedule") ;
	if (nDTSTAMP != null && nDTSTAMP.length > 0 ) {
		extraInfoSchedule += " "
			+ reminderfox.core.getBundle().getString("rf.schedule.asof") + "  " + nDTSTAMP;
	}
	if ( nSEQ != null ) {
		extraInfoSchedule += " [# " + nSEQ + "]" + ret ;
	}
	extraInfoSchedule += mInvitation + ret;

	// <<<----  process PERSONS infos  ------


	switch (aMethod) {
		case "REQUEST":
		case "PUBLISH": { // this is send to ATTENDEE(s), can be
											// - new
											// - revised

			// ICS containing "ATTENDEE" is an invitation !
			var anInvitation = ((reminder.extraInfo.indexOf("ATTENDEE:") > -1 )
				|| (reminder.extraInfo.indexOf("ATTENDEE;") > -1)) ? true : false;

			if (anInvitation) { // check for invitation
						// ++++++++++++ Google Calendar Special  ++++++++++++
						if ( reminder.notes != null && reminder.notes.length > 0 ) {
							var iGMail = reminder.notes.toLowerCase().indexOf("http://www.google.com/calendar/event?action=view");
							if ( iGMail != -1) {
								var iGMail1 = reminder.notes.toLowerCase().indexOf(" ",iGMail);
								var httpGMail = reminder.notes.substring(iGMail,iGMail1);
								reminder.url = httpGMail;
							}
						}
						// ++++++++++++ Google Calendar Special  ++++++++++++

				if (addOption != "" )
					extraInfoSchedule += "*" +reminderfox.core.getBundle().getString("rf.schedule.options") +
					"* ______________________" + ret + addOption + ret;

				if (iSchComment != null )
					extraInfoSchedule += "*" +reminderfox.core.getBundle().getString("rf.schedule.comment") +
					"* ______________________" + ret + iSchComment + ret;

				if (reminder.notes != null && reminder.notes.length > 0 ) {
					reminder.notes = "> " + reminder.notes.replace(new RegExp(/\n/g),"\n>");
				}

				if (reminder.notes != null ) {
					extraInfoSchedule +=  "*" +reminderfox.core.getBundle().getString("rf.schedule.description") +
					"*  ______________________" + ret
					+  reminder.notes + ret;
				}
				reminder.notes = extraInfoSchedule;

				// add 'Category' item signifying that this is an invitation
				var categoryItem = reminderfox.core.getBundle().getString("rf.mail.ical.send.subject.invitation"); //	"invitation";
				reminder = reminderfox.util.addCategory4Import (reminder, categoryItem);

				return msgReturn;

			} else {  // this is NOT an invitation
			}
			break;

		} //case:REQUEST" & "PUBLISH"

		case "REPLY": {	// this is send back to ORGANIZER

			reminder.notes = extraInfoSchedule;
			return msgReturn;
			break;
		}  //case "REPLY"

		case "CANCEL": {  // this 'cancel's the local event/invitation

			reminder.notes = extraInfoSchedule;
			return msgReturn;
			break;
		}  //case "CANCEL"

	}  // switch (aMethod)
	return msgReturn;
};


/**
 *    With the selected Reminder/Todo's   <br>
 *   'Export to file' or  'Send a Mail' as an INVITATION/SCHEDULE
 *
 * @param  event   has '.id' for selected ReminderTyp, if 'null' called from function
 * @param  _selectedTyp    // 'invite'   | 'info'
 * @param  _reminderTyp    // 'reminder' | 'todo'
 * @param  _Reminders      //
 *
 * @return
 */
reminderfox.iCal.exportOrSend = function (event, _selectedTyp, _reminderTyp, _Reminders) {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	var methodTyp = "PUBLISH";  // default=PUBLISH ... other METHOD:PUBLISH, REQUEST, REPLY
	var attendees = null;
	var organizer = null;
	var selectedTyp;  // 'invite'   | 'info'
	var reminderTyp;  // 'reminder' | 'todo'
	var eventLength;  // use to show # of events in XUL title

	// function call (eg. from 'View')
	if (event == null) {
		selectedTyp = _selectedTyp;
		reminderTyp = _reminderTyp;
		reminderfox.iCal.selectedReminders = _Reminders;

		if (reminderTyp == 'reminder') eventLength = reminderfox.iCal.selectedReminders.length;
		if (reminderTyp == 'todo')     eventLength = reminderfox.iCal.selectedReminders[0].length;

	} else {

		// called from ReminderList popup menu
		if (event.currentTarget.id == "treechildren-contextmenu-sendReminder")  {
			selectedTyp = 'invite';
			reminderTyp = 'reminder';

			reminderfox.iCal.selectedReminders = getAllSelectedReminders();
			var eventLength = reminderfox.iCal.selectedReminders.length;
			if (eventLength == 0) return;  // no reminders selected!

			if (reminderfox.iCal.selectedReminders.length > 1)  {
				selectedTyp = 'info';	// only 'single' events can be send as invitation
			}
		}

		if (event.currentTarget.id == "treechildren-contextmenu-sendTodos") {
			  selectedTyp = 'info';
			  reminderTyp = 'todo';

			reminderfox.iCal.selectedReminders = reminderfox.mail.getSelectedTodos();

			if (reminderfox.iCal.selectedReminders == null) {
				reminderfox.core.statusSet (reminderfox.core.getBundle().getString("rf.schedule.resendwithDialog")) // ("No Todo's selected!");
				return;
			}

			var eventLength = reminderfox.iCal.selectedReminders[0].length;
		}
	 }

	 if (selectedTyp == 'invite') {

		/*  have to check if the selected "Reminder" was already send as invitation/schedule;
		 *  that's the case if ORGANIZER is already included in "this.selectedReminders"
		 */
		if (reminderfox.iCal.selectedReminders[0].extraInfo != null) {
			if (reminderfox.iCal.selectedReminders[0].extraInfo.indexOf("ORGANIZER") != -1) {
				
				var title = reminderfox.core.getBundle().getString("rf.schedule.wasSend");
				var msg = reminderfox.core.getBundle().getString("rf.schedule.wasSend")
					+ "\n"	+ reminderfox.core.getBundle().getString("rf.schedule.resendwithDialog")
					+ "\n\n" + reminderfox.core.getBundle().getString("rf.schedule.sendAsEvent")
					+ "\n" + "[Delete] To cancel previous schedule." // reminderfox.core.getBundle().getString("rf.schedule.sendAsEvent");
				var key0 = reminderfox.core.getBundle().getString("rf.button.ok")
				var key1 = reminderfox.core.getBundle().getString("rf.button.cancel");
				var key2 = "Delete";// reminderfox.core.getBundle().getString("rf.schedule.okDelete");
				var mode = reminderfox.util.ConfirmEx(title, msg, key0, key1, key2);

				if (mode == 1) return;

				if (mode == 2) {  // this is the 'delete schedule' part
					var extraInfoArrayInv = reminderfox.core.extraInfoArray(reminderfox.iCal.selectedReminders[0]);
					var extraInfoArray = new Array ();  var j=0;

					for (var i=0; i < extraInfoArrayInv.length; i++) {
						if (extraInfoArrayInv[i].indexOf("ORGANIZER")> -1 ||
							extraInfoArrayInv[i].indexOf("ATTENDEE")  > -1 ||
					/*		extraInfoArrayInv[i].indexOf("DTSTAMP")   > -1 ||    do not erase this!!  */
							extraInfoArrayInv[i].indexOf("STATUS")    > -1 ||
							extraInfoArrayInv[i].indexOf("CLASS")     > -1 ||
							extraInfoArrayInv[i].indexOf("COMMENT")   > -1 ||
							extraInfoArrayInv[i].indexOf("SEQUENCE")  > -1 )
							{
							} else {
							extraInfoArray[j] = extraInfoArrayInv[i]; j++;
						}
					}
					// delete the 'invitation' category also
					var xCategories = reminderfox.iCal.selectedReminders[0].categories;

					var categoryItem = reminderfox.core.getBundle().getString("rf.mail.ical.send.subject.invitation"); //	"invitation";
					if (xCategories.indexOf(categoryItem) > -1) {
						reminderfox.iCal.selectedReminders[0].categories = xCategories.replace(categoryItem, '');
					}

					// write the selectedReminder back to currentReminder without 'extra' strings
					reminderfox.iCal.selectedReminders[0].extraInfo= extraInfoArray.join("\n");
					var reminderTodos = null;
					reminderfox.util.addMultipleEvents(reminderfox.iCal.selectedReminders, reminderTodos)

					reminderfox.util.PromptAlert(reminderfox.core.getBundle().getString("rf.schedule.hasDeleted")
						  + "\n\n" + reminderfox.core.getBundle().getString("rf.schedule.checkDeleted"));

					return;
				}
				selectedTyp ='info';  // was already send, don't allow to send as 'invite'
			}
		}
	} // if invite


	// Dialog   'Send' or 'Export'  Reminder(s)
	var results = {	organizer: null,
					attendees: null,
					mode: null,
					selectedTyp: selectedTyp,
					reminderTyp: reminderTyp,
					eventLength: eventLength,
					schComment: ""};
	window.openDialog("chrome://reminderfox/content/mail/invitation.xul",
					"reminderFox-set-invitationmailAppString", "chrome,resizable,modal", results);

	// if 'CANCEL' or 'close window' pressed -->> terminate
	if (results.mode == 'CANCEL') return;

	if (results.mode == 'xExport') {			// export to ICS file
		reminderfox.mail.sendAsItIs (reminderfox.iCal.selectedReminders, results.organizer, results.mode, reminderTyp)
		return;
	}


	// send "as-it-is" ===> PUBLISH
	if (results.selectedTyp == 'info') {

		var attendees = null;
		if (reminderTyp == 'reminder') {
		// this.selectedReminders[0].url $[1] = [string] "mailto:abc@xyz.de"
			if ((reminderfox.iCal.selectedReminders.length == 1) && (!!reminderfox.iCal.selectedReminders[0].url)){
				 if (reminderfox.iCal.selectedReminders[0].url.indexOf('mailto:') == 0)
					  attendees = reminderfox.iCal.selectedReminders[0].url.substring(7);
			}
			reminderfox.mail.sendEventNow(reminderfox.iCal.selectedReminders, "", 'ICS' /*methodTyp*/, results.organizer, results.attendees, reminderTyp);
		}

		if (reminderTyp == 'todo') {
		// this.selectedReminders[0][0].url $[1] = [string] "mailto:abc@xyz.de"
			if ((reminderfox.iCal.selectedReminders[0].length == 1) && (!!reminderfox.iCal.selectedReminders[0][0].url)){
				 if (reminderfox.iCal.selectedReminders[0][0].url.indexOf('mailto:') == 0)
					  attendees = reminderfox.iCal.selectedReminders[0][0].url.substring(7);
			}
			reminderfox.mail.sendEventNow("", reminderfox.iCal.selectedReminders,  'ICS' /*methodTyp*/, results.organizer, results.attendees, reminderTyp);
		}
		return;
	}

	// send  INVITATION/SCHEDULE  ===> REQUEST
	if (results.selectedTyp == 'invite') {methodTyp = 'REQUEST';}

	organizer = results.organizer;
	if (organizer == "") {
		reminderfox.util.PromptAlert(reminderfox.core.getBundle().getString("rf.schedule.organizer.required")
		+ "\n" + reminderfox.core.getBundle().getString("rf.schedule.organizer.mailAdr"));
		return;
	}

	if (results.attendees != null && results.attendees != "") {
		methodTyp = "REQUEST";	  // attendee(s) --> REQUEST
		} else {
		methodTyp = "PUBLISH";
	}
	//  add invitation info to 'extraInfo' item
	var originalReminder = reminderfox.iCal.selectedReminders[0];
	var updatedReminder = reminderfox.core.cloneReminderFoxEvent(reminderfox.iCal.selectedReminders[0] );


	updatedReminder.extraInfoArray = reminderfox.core.extraInfoArray  (updatedReminder);
	
	// replace DTSTAMP (current date/time in 'Z')
	var currentDate = new Date();
	var tzOffset = new Date().getTimezoneOffset();
	currentDate.setMinutes( currentDate.getMinutes() + tzOffset );
	var dtStamp = reminderfox.date.getDateAsString(currentDate)+ "Z";
	reminderfox.core.extraInfoAdd (updatedReminder.extraInfoArray, "DTSTAMP", dtStamp);
	// add SEQUENCE=1 
	reminderfox.core.extraInfoAdd (updatedReminder.extraInfoArray, "SEQUENCE", 1);

	// XXXXX  updatedReminder.extraInfo = updatedReminder.extraInfoArray.join("\\n") + "\\n";




	//2008-12-16	clear trailing/leading spaces
	var organizerName = reminderfox.util.trim(organizer.split("<")[0]);
	var organizerMail = reminderfox.util.trim(organizer.split("<")[1]);

	if (organizerMail == null) {
			organizerMail = organizerName;
			organizerName ="";
	}
	if (organizerMail.charAt(organizerMail.length-1) == ">") {
		organizerMail = organizerMail.substring(0,organizerMail.length-1)
	}
	reminderfox.core.extraInfoAdd (updatedReminder.extraInfoArray, 
		"ORGANIZER", "MAILTO:" + organizerMail, "CN=" + organizerName);


	// add ATTENDEE(s)
	if (methodTyp == "REQUEST") {
		attendees = results.attendees.replace(/(^\s+)|(\s+$)/g, '');

		//  ATTENDEE;RSVP=TRUE:MAILTO:abcde@fghi.xx
		var attendeeArray = attendees.split(",");
		for ( var i = 0; i < attendeeArray.length; i++  ) {
			if (attendeeArray[i] == "") {break;}
			var attendeeMail ="";
			var attendeeName ="";

			var attendeeX = attendeeArray[i].split("<");
			if (attendeeX.length == 1) {  // only mailAdr
				var attendeeMail = attendeeX[0].replace(/(^\s+)|(\s+$)/g, '').replace(">", '');
			} else {					 // mailName and adr
				var attendeeName = attendeeX[0].replace(/(^\s+)|(\s+$)/g, '');
				var attendeeMail = attendeeX[1].replace(/(^\s+)|(\s+$)/g, '').replace(">", '');
			}
			reminderfox.core.extraInfoAdd (updatedReminder.extraInfoArray, 
					"ATTENDEE", "MAILTO:" + attendeeMail, "CN=" + attendeeName
						+ ";RSVP=TRUE;PARTSTAT=NEEDS-ACTION;ROLE=REQ-PARTICIPANT");
		}
	}


	// add 'comment'  from invitation.xul dialog
	if (results.schComment != "") {
		/// updatedReminder.extraInfo += "\\nCOMMENT:" + results.schComment;
		
		reminderfox.core.extraInfoAdd (updatedReminder.extraInfoArray, "COMMENT", results.schComment);
	}
	
	// build new extraItem string from updatedReminder.extraInfoArray
	updatedReminder.extraInfo = updatedReminder.extraInfoArray.join("\\n");
	

	
	// add other items
	reminderfox.iCal.processExtraScheduleInfo(updatedReminder, "", methodTyp );

	// add category
	var categoryItem = reminderfox.core.getBundle().getString("rf.mail.ical.send.subject.invitation"); //	"invitation";
	 updatedReminder = reminderfox.util.addCategory4Import (updatedReminder, categoryItem);

	//  REPLACE the current entry
	removeListItemsForReminder(originalReminder);
	// remove from model list
	var reminders = reminderfox.core.getReminderEvents();
	for( var i = 0; i < reminders.length; i++) {
		if ( reminders[i].id == originalReminder.id ) {
			reminderfox.core.removeElement(reminders, i);
			break;
		}
	}

	// add reminder in sorted order...
	reminders = reminderfox.core.getReminderEvents();
	if ( updatedReminder.remindUntilCompleted != reminderfox.core.consts.REMIND_UNTIL_COMPLETE_NONE ) {
		updatedReminder.remindUntilCompleted = reminderfox.core.consts.REMIND_UNTIL_COMPLETE_TO_BE_MARKED;
	}
	var sortedIndex = reminderfox.core.getSortedIndexOfNewReminder( reminders, updatedReminder, false );
	reminderfox.core.insertIntoArray( reminders, updatedReminder, sortedIndex );

	updatedReminder.snoozeTime = null;
	createListItem(updatedReminder);

	var currentDate = reminderfox.datePicker.gSelectedDate;
	if ( currentDate == null ) {
		currentDate = new Date();
	}

	var newDate = new Date();
	if ( updatedReminder.alarm != null ) {
		updatedReminder.alarmLastAcknowledge = newDate.getTime();
		// if there's a snooze... and lastack > snooze, erase snooze...
		// this means...  well it should be if date changed at all, then need to clear snooze...
	}
	updatedReminder.lastModified = reminderfox.date.getDateAsString(newDate );
	var reminder = reminderfox.core.processReminderDescription(updatedReminder,  currentDate.getFullYear(), false);
	var modified = updateReminderInList( reminder );
	sortReminderIfNeccessary( updatedReminder.id);

	refreshCalendar();
	modifiedReminders();

	reminderfox.iCal.selectedReminders[0] = updatedReminder; // need array
	reminderfox.mail.sendEventNow(reminderfox.iCal.selectedReminders, "", methodTyp, organizer, attendees, selectedTyp);
};




/**
 *  Set the file for  'export' of reminders/todos
 *  .. and set prefs for it
 
 */
reminderfox.iCal.getExportFile = function () {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	 var currentFileName	 = document.getElementById("exportFile").value;
	 var windowText			= reminderfox.core.getBundle().getString("rf.export.send.title");	// "Set / Select file to export reminder(s)";
	 var filterName			= "Event File (ICS)";
	 var defaultExtension	= "ICS";
	 var rmFx_exportFile		= "exportEventsFile"	 // set as prefs 'extensions.reminderFox.exportEventsFile'

	 var file = reminderfox.core.filePicker (windowText, filterName, defaultExtension,
						currentFileName /*defaultString*/, rmFx_exportFile /*prefsName*/, 1 /*mode = modeSave*/)

	 if (file != null) {
		reminderfox.core.consts.reminderFox_prefsBranch.setCharPref(rmFx_exportFile, file.path);
		document.getElementById("exportFile").setAttribute("value", file.path);
	 }
};


/**
 *  Parse   ORGANIZER;CN=name:MAILTO:mailadr@abc.xx
 *  
 *  @returns  name<mailadr@abc.xx>
 */
 reminderfox.iCal.getMailAddress = function (icsString) {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	var persName="";
	var persMail="";
	var fndIndex = icsString.toUpperCase().lastIndexOf( "MAILTO:");
	if (fndIndex != -1) {
		persMail = icsString.substring( fndIndex + 7);
	} else {
		var colnIndex = icsString.lastIndexOf(":");
		if (colnIndex != -1) {
			if (icsString.substring( colnIndex).indexOf("@") != -1) {
				persMail = icsString.substring( colnIndex + 1);  }
		}
	}
	var fndIndex = icsString.toUpperCase().indexOf( "CN=" );
	if (fndIndex != -1) {
		var fndString = icsString.substring( fndIndex + 3);
		var colnIndex = fndString.indexOf(":");
		var semkiIndex = fndString.indexOf(";");

		if ((semkiIndex != -1) && (semkiIndex < colnIndex))  {
			persName = fndString.substring( 0, semkiIndex);
		} else { persName = fndString.substring( 0, colnIndex);  }
	}

	if (persName == "") { return '<' + persMail + '>';}
	
	return persName + '<' + persMail +'>';
};



/**
 *   RmFx reschedule an event
 *   
 *   @see  http://tools.ietf.org/html/rfc2446#section-3.2.2.1 Rescheduling an Event
 */
reminderfox.iCal.reschedule = function(originalReminder, currentReminder) {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	if (originalReminder.extraInfo == null) return;

	var extraInfoOrginal = reminderfox.core.extraInfoArray (originalReminder);
	var extraInfoCurrent = reminderfox.core.extraInfoArray (currentReminder);

	var currentATT = reminderfox.core.extraInfo( extraInfoOrginal,"ATTENDEE");
	if (currentATT == null || currentATT.length == 0)  { return;}

	// OK  it's a Invitation/Schedule .. rescheduling requirements:
	//   1. is the ORGANIZER one of my accounts?  Maybe hard with FX!!
	//   2. A rescheduled event involves time or recurrence intervals
	//      and possibly the location or description

	// ad 1.  check ORGANIZER
	//   currentORG = "ORGANIZER;CN=name:MAILTO:mailadr@abc.xx"
	//   myAccounts = "name<mailadr@xyz.xx>,name2<mailadr2@mmm.zz>"

	var currentORG = reminderfox.core.extraInfo( extraInfoOrginal,"ORGANIZER", ";");
	if (currentORG.length == 0)  { return;}  // no ORGANIZER
	var fndIndex = currentORG.toUpperCase().lastIndexOf("MAILTO:");

	if (reminderfox.msgnr.myMailIDs().toUpperCase().indexOf(currentORG
		.substring( fndIndex + 7).toUpperCase()) == -1 ) {return;} // not the ORGANIZER -- no Invitation/Schedule

	// ORGANIZER ok
	var fromAddress = reminderfox.iCal.getMailAddress(currentORG);

  // ad 2. check if parameters changed

	if (currentReminder.endDate == null  && originalReminder.endDate != null) {
		currentReminder.endDate = originalReminder.endDate;
	}

	var dateChange =
			originalReminder.date.toGMTString()    != currentReminder.date.toGMTString()
		|| originalReminder.endDate.toGMTString() != currentReminder.endDate.toGMTString();

	var recurChange = (
			originalReminder.recurrence.byDay      != currentReminder.recurrence.byDay
		|| originalReminder.recurrence.endDate    != currentReminder.recurrence.endDate
		|| originalReminder.recurrence.interval   != currentReminder.recurrence.interval
		|| originalReminder.recurrence.type       != currentReminder.recurrence.type);

	var sumChange  = originalReminder.summary != currentReminder.summary;

	var locChange  = originalReminder.location != currentReminder.location;

	if ( !(dateChange || recurChange || sumChange|| locChange ) ) {return;}  // no Change

  //  ORGANIZER 'rescheduled' the event
  // read ORGANIZER   from reminder and set it to 'fromAddress'
  // read ATTENDEE(s) from reminder and set it to 'attendees'
  // parameter to be updated:
  //   - DTSTAMP
  //   - SEQUENCE
  //   - SUMMARY/DESCRIPTION
  //   - LASTMODIFIED ????


	var aExtraInfo  = reminderfox.core.extraInfoArray(currentReminder);

	// for ATTENDEE(s) have to build the to=mailString and all ATT need to called RSVP!!
	var attendeeString =""; // for iCal ATTENDEE:
	var attendees ="";      // for mail compose to=

	for (var i=0; i< aExtraInfo.length; i++) {
		if (aExtraInfo[i].indexOf("ATTENDEE") == 0) {

			var attendeeName="";
			var attendeeMail="";
			var fndIndex = aExtraInfo[i].toUpperCase().lastIndexOf( "MAILTO:");
			  if (fndIndex != -1) {
					attendeeMail = aExtraInfo[i].substring( fndIndex + 7);
			  } else {
				var colnIndex = aExtraInfo[i].lastIndexOf(":");
				if (colnIndex != -1) {
					if (aExtraInfo[i].substring( colnIndex).indexOf("@") != -1) {
						attendeeMail = aExtraInfo[i].substring( colnIndex + 1);  }
				}
			}
			  var fndIndex = aExtraInfo[i].toUpperCase().indexOf( "CN=" );
			  if (fndIndex != -1) {
				var fndString = aExtraInfo[i].substring( fndIndex + 3);
				var colnIndex = fndString.indexOf(":");
				var semkiIndex = fndString.indexOf(";");

				if ((semkiIndex != -1) && (semkiIndex < colnIndex))  {
					attendeeName = fndString.substring( 0, semkiIndex);
				} else {attendeeName = fndString.substring( 0, colnIndex);  }
			  }
			if (attendeeName == "") {attendees +=  '<' + attendeeMail +'>,';
				} else {
				attendees += attendeeName + '<' + attendeeMail +'>,';
			}

			var attendeeString ="ATTENDEE";
			if (attendeeName != "") {attendeeString += ";CN=" + attendeeName}
			if (attendeeMail != "") {attendeeString += ";RSVP=TRUE;PARTSTAT=NEEDS-ACTION;ROLE=REQ-PARTICIPANT:MAILTO:" + attendeeMail}
			aExtraInfo[i] = attendeeString;
		}
	}
	reminderfox.core.extraInfoReplace (aExtraInfo, "SEQUENCE", reminderfox.core.extraInfo(aExtraInfo, "SEQUENCE")*1 + 1);
	reminderfox.core.extraInfoReplace (aExtraInfo, "DTSTAMP", reminderfox.date.setDateTimeZ());
	reminderfox.core.extraInfoReplace (aExtraInfo, "COMMENT", "");	  // erase old

	currentReminder.extraInfo = aExtraInfo.join("\\n");
	// delete double 'CR'
	currentReminder.extraInfo = currentReminder.extraInfo.replace(new RegExp(/\\n\\n/g),"\\n");

	var chText =  (dateChange  ? " '" + reminderfox.core.getBundle().getString("rf.add.mail.message.datetime")   + "' "  : "")
					+ (recurChange ? " '" + reminderfox.core.getBundle().getString("rf.add.mail.message.reccurance") + "' "  : "")
					+ (sumChange   ? " '" + reminderfox.core.getBundle().getString("rf.schedule.description")        + "' "  : "")
					+ (locChange   ? " '" + reminderfox.core.getBundle().getString("rf.mail.ical.send.location")     + "' "  : "");

	var methodTyp = "REQUEST";
	var newNotes = "[iCal] " + methodTyp + ": " + reminderfox.core.getBundle().getString("rf.schedule.statusOf")
		+ ": " + reminderfox.date.parseDateTimes(reminderfox.core.extraInfo(aExtraInfo,"DTSTAMP"))
		+ " [#" + reminderfox.core.extraInfo(aExtraInfo,"SEQUENCE") +"]\n"
		+ "*" + reminderfox.core.getBundle().getString("rf.schedule.statusChanged")+"*  " + chText +"\n\n";
	currentReminder.notes = newNotes + currentReminder.notes
	var reminders = new Array();
	reminders[0] = currentReminder;
	var iCalInvite = true;
	reminderfox.mail.sendEventNow(reminders, "", methodTyp, fromAddress, attendees, iCalInvite);
};


/**
 * Change "Edit Reminder" dialog for iCal invitation
 * 
 * @param {Object} currentReminder
 */
reminderfox.iCal.inviteStatus = function (currentReminder) {
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
// var extraInfoOrginal = reminderfox.core.extraInfoArray (currentReminder);

	var currentORG = reminderfox.core.extraInfo( reminderfox.core.extraInfoArray (currentReminder),"ORGANIZER");
	if (currentORG.length == 0)  { return;}  // no ORGANIZER

	var fndIndex = currentORG.toUpperCase().lastIndexOf( "MAILTO:" );
	if (reminderfox.msgnr.myMailIDs().toUpperCase()
		.indexOf(currentORG.substring( fndIndex + 7).toUpperCase()) == -1) {

			document.getElementById("newReminderText").setAttribute("disabled", "true");

			document.getElementById("reminderFox-all-day").setAttribute("disabled", "true");

			document.getElementById("reminderFox-timeList").setAttribute("disabled", "true");
			document.getElementById("yearlist").setAttribute("disabled", "true");
			document.getElementById("monthlist").setAttribute("disabled", "true");
			document.getElementById("daylist").setAttribute("disabled", "true");

			document.getElementById("reminderFox-endTimeList").setAttribute("disabled", "true");
			document.getElementById("endyearlist").setAttribute("disabled", "true");
			document.getElementById("endmonthlist").setAttribute("disabled", "true");
			document.getElementById("enddaylist").setAttribute("disabled", "true");

			document.getElementById("datepickerToggle").setAttribute("disabled", "true");
			document.getElementById("datepickerEndToggle").setAttribute("disabled", "true");

			document.getElementById("location").setAttribute("disabled", "true");
			document.getElementById("reminderFox-repeat").setAttribute("disabled", "true");
	}
};



// reminderfox.iCal.test   NOT TESTED after REFRACTOR !!!   gW
/**
 * //gWTEST reminderfox.iCal.test	 TEST  TEST  TEST  TEST  TEST  TEST
 *  test purpose only:  copy a BEGIN:CALENDAR ... END:CALENDAR
 *   to clipboard and call this function to process /simulate
 *   an 'iCal import form email'
 */
/*--------------------
reminderfox.iCal.test = function (){
// ------------- --------------------- --------------------- ---------------- -
	var clip = Components.classes["@mozilla.org/widget/clipboard;1"].
	getService(Components.interfaces.nsIClipboard);
	if (!clip) return false;

	var trans = Components.classes["@mozilla.org/widget/transferable;1"].
		createInstance(Components.interfaces.nsITransferable);
	if (!trans) return false;
	trans.addDataFlavor("text/unicode");

	clip.getData(trans,clip.kGlobalClipboard);
	var str = new Object();
	var strLength = new Object();
	trans.getTransferData("text/unicode",str,strLength);

	if (str) str = str.value.QueryInterface(Components.interfaces.nsISupportsString);
	if (str) pastetext = str.data.substring(0,strLength.value / 2);

	var mInvitation = "[iCal] Schedule copied from Clipboard";
	this.reminderfox_Method ="CLIPBOARD";

	this.tagging(this.getiCalFromString(pastetext, mInvitation));
	return true;
};
----------------*/
